// src/internal/adapter.ts
import { WebNextRequest, WebNextResponse } from "next/dist/esm/server/base-http/web";

// src/internal/manifest-provider.ts
var ManifestProvider = class {
};
var IgnextManifestProvider = class extends ManifestProvider {
  options;
  constructor(options) {
    super();
    this.options = options;
  }
  getPrerenderManifest() {
    return this.options.prerenderManifest;
  }
  getPagesManifest() {
    return this.options.pagesManifest;
  }
  getAppPathsManifest() {
    return this.options.appPathsManifest;
  }
  getRoutesManifest() {
    return this.options.routesManifest;
  }
  getFontManifest() {
    return this.options.fontManifest;
  }
  getServerComponentManifest() {
    return this.options.serverComponentManifest;
  }
  getServerCSSManifest() {
    return this.options.serverCSSManifest;
  }
  getFontLoaderManifest() {
    return this.options.fontLoaderManifest;
  }
  getBuildId() {
    return this.options.buildId;
  }
  getMiddlewareManifest() {
    return this.options.middlewareManefest;
  }
};

// src/internal/server.ts
import { parse as parseQs } from "querystring";
import { format as formatUrl2, parse as parseUrl3 } from "url";
import { getProperError as getProperError3 } from "next/dist/esm/lib/is-error";
import { getCookieParser, setLazyProp } from "next/dist/esm/server/api-utils";
import { getClonableBody } from "next/dist/esm/server/body-streams";
import {
  addRequestMeta as addRequestMeta2,
  getRequestMeta as getRequestMeta3
} from "next/dist/esm/server/request-meta";
import WebResponseCache from "next/dist/esm/server/response-cache/web";
import { TEMPORARY_REDIRECT_STATUS } from "next/dist/esm/shared/lib/constants";
import { getHostname } from "next/dist/esm/shared/lib/get-hostname";
import { detectDomainLocale as detectDomainLocale2 } from "next/dist/esm/shared/lib/i18n/detect-domain-locale";
import { getLocaleRedirect } from "next/dist/esm/shared/lib/i18n/get-locale-redirect";
import { normalizeLocalePath as normalizeLocalePath4 } from "next/dist/esm/shared/lib/i18n/normalize-locale-path";
import {
  getSortedRoutes,
  isDynamicRoute as isDynamicRoute3
} from "next/dist/esm/shared/lib/router/utils";
import {
  normalizeAppPath as normalizeAppPath2,
  normalizeRscPath
} from "next/dist/esm/shared/lib/router/utils/app-paths";
import { getNextPathnameInfo as getNextPathnameInfo2 } from "next/dist/esm/shared/lib/router/utils/get-next-pathname-info";
import { parseUrl as parseUrlUtil } from "next/dist/esm/shared/lib/router/utils/parse-url";
import { removePathPrefix } from "next/dist/esm/shared/lib/router/utils/remove-path-prefix";
import { getRouteMatcher } from "next/dist/esm/shared/lib/router/utils/route-matcher";
import { getRouteRegex } from "next/dist/esm/shared/lib/router/utils/route-regex";
import {
  DecodeError as DecodeError3,
  NormalizeError as NormalizeError2,
  normalizeRepeatedSlashes as normalizeRepeatedSlashes2
} from "next/dist/esm/shared/lib/utils";

// src/internal/logger.ts
var Logger = class {
  constructor(quite) {
    this.quite = quite;
  }
};
var ConsoleLogger = class extends Logger {
  logError(error) {
    if (this.quite)
      return;
    console.error(error);
  }
};

// src/internal/page-checker.ts
import { normalizeLocalePath } from "next/dist/esm/shared/lib/i18n/normalize-locale-path";
import { denormalizePagePath } from "next/dist/esm/shared/lib/page-path/denormalize-page-path";
import { normalizePagePath } from "next/dist/esm/shared/lib/page-path/normalize-page-path";
var PageChecker = class {
};
var IgnextPageChecker = class extends PageChecker {
  constructor(manifestProvider, nextConfig) {
    super();
    this.manifestProvider = manifestProvider;
    this.nextConfig = nextConfig;
  }
  async hasPage(pathname) {
    try {
      pathname = denormalizePagePath(normalizePagePath(pathname));
    } catch (error) {
      console.error(error);
      return false;
    }
    const checkManifest = (manifest) => {
      let curPath = manifest[pathname];
      if (!manifest[curPath] && this.nextConfig.i18n?.locales) {
        const manifestNoLocales = {};
        for (const key of Object.keys(manifest)) {
          manifestNoLocales[normalizeLocalePath(key, this.nextConfig.i18n?.locales).pathname] = this.manifestProvider.getPagesManifest()[key];
        }
        curPath = manifestNoLocales[pathname];
      }
      return curPath;
    };
    let pagePath;
    if (this.manifestProvider.getAppPathsManifest()) {
      pagePath = checkManifest(this.manifestProvider.getAppPathsManifest());
    }
    if (!pagePath) {
      pagePath = checkManifest(this.manifestProvider.getPagesManifest());
    }
    if (!pagePath) {
      return false;
    }
    return true;
  }
};

// src/internal/renderer.ts
import { format as formatUrl, parse as parseUrl } from "url";
import * as Log from "next/dist/esm/build/output/log";
import { getProperError } from "next/dist/esm/lib/is-error";
import { getRedirectStatus } from "next/dist/esm/lib/redirect-status";
import { checkIsManualRevalidate } from "next/dist/esm/server/api-utils";
import { byteLength } from "next/dist/esm/server/api-utils/web";
import { renderToHTMLOrFlight } from "next/dist/esm/server/app-render";
import {
  NoFallbackError
} from "next/dist/esm/server/base-server";
import { generateETag } from "next/dist/esm/server/lib/etag";
import { renderToHTML } from "next/dist/esm/server/render";
import RenderResult from "next/dist/esm/server/render-result";
import {
  getRequestMeta
} from "next/dist/esm/server/request-meta";
import {
  setRevalidateHeaders
} from "next/dist/esm/server/send-payload";
import { isBlockedPage } from "next/dist/esm/server/utils";
import { isBot } from "next/dist/esm/server/web/spec-extension/user-agent";
import {
  NEXT_BUILTIN_DOCUMENT,
  STATIC_STATUS_PAGES
} from "next/dist/esm/shared/lib/constants";
import { normalizeLocalePath as normalizeLocalePath2 } from "next/dist/esm/shared/lib/i18n/normalize-locale-path";
import { denormalizePagePath as denormalizePagePath2 } from "next/dist/esm/shared/lib/page-path/denormalize-page-path";
import { normalizeAppPath } from "next/dist/esm/shared/lib/router/utils/app-paths";
import escapePathDelimiters from "next/dist/esm/shared/lib/router/utils/escape-path-delimiters";
import { isDynamicRoute } from "next/dist/esm/shared/lib/router/utils/is-dynamic";
import { removeTrailingSlash } from "next/dist/esm/shared/lib/router/utils/remove-trailing-slash";
import {
  DecodeError,
  execOnce,
  MissingStaticPage,
  NormalizeError,
  normalizeRepeatedSlashes
} from "next/dist/esm/shared/lib/utils";
var WrappedBuildError = class extends Error {
  innerError;
  constructor(innerError) {
    super();
    this.innerError = innerError;
  }
};
var Renderer = class {
  customErrorNo404Warn = execOnce(() => {
    Log.warn(
      `You have added a custom /_error page without a custom /404 page. This prevents the 404 page from being auto statically optimized.
See here for info: https://nextjs.org/docs/messages/custom-error-no-custom-404`
    );
  });
  appPathRoutes;
  nextConfig;
  renderOpts;
  pageChecker;
  manifestProvider;
  responseCache;
  logger;
  dynamicRoutes;
  hasMiddleware;
  constructor(options) {
    this.nextConfig = options.nextConfig;
    this.renderOpts = options.renderOpts;
    this.pageChecker = options.pageChecker;
    this.manifestProvider = options.manifestProvider;
    this.responseCache = options.responseCache;
    this.logger = options.logger;
    this.dynamicRoutes = options.dynamicRoutes;
    this.hasMiddleware = options.hasMiddleware;
    this.appPathRoutes = this.getAppPathRoutes();
  }
  async render(request, res, pathname, query = {}, parsedUrl, internalRender = false) {
    if (!pathname.startsWith("/")) {
      console.warn(
        `Cannot render page with path "${pathname}", did you mean "/${pathname}"?. See more info here: https://nextjs.org/docs/messages/render-no-starting-slash`
      );
    }
    if (this.renderOpts.customServer && pathname === "/index" && !await this.pageChecker.hasPage("/index")) {
      pathname = "/";
    }
    if (isBlockedPage(pathname)) {
      return this.render404(request, res, parsedUrl);
    }
    return this.pipe(async (ctx) => this.renderToResponse(ctx), {
      req: request,
      res,
      pathname,
      query
    });
  }
  async render404(request, res, parsedUrl, setHeaders = true) {
    const { pathname, query } = parsedUrl ? parsedUrl : parseUrl(request.url, true);
    if (this.nextConfig.i18n) {
      query.__nextLocale = query.__nextLocale ?? this.nextConfig.i18n.defaultLocale;
      query.__nextDefaultLocale = query.__nextDefaultLocale ?? this.nextConfig.i18n.defaultLocale;
    }
    res.statusCode = 404;
    return this.renderError(
      void 0,
      request,
      res,
      pathname,
      query,
      setHeaders
    );
  }
  async renderError(error, request, res, pathname, query = {}, setHeaders = true) {
    if (setHeaders) {
      res.setHeader(
        "Cache-Control",
        "no-cache, no-store, max-age=0, must-revalidate"
      );
    }
    return this.pipe(
      async (ctx) => {
        const response = await this.renderErrorToResponse(ctx, error);
        return response;
      },
      { req: request, res, pathname, query }
    );
  }
  async getFallbackErrorComponents() {
    return void 0;
  }
  async getStaticPaths({
    pathname
  }) {
    const staticPaths = void 0;
    const fallbackField = this.manifestProvider.getPrerenderManifest().dynamicRoutes[pathname]?.fallback;
    return {
      staticPaths,
      fallbackMode: typeof fallbackField === "string" ? "static" : fallbackField === null ? "blocking" : fallbackField
    };
  }
  getAppPathRoutes() {
    const appPathRoutes = {};
    for (const entry of Object.keys(
      this.manifestProvider.getAppPathsManifest() ?? {}
    )) {
      const normalizedPath = normalizeAppPath(entry) || "/";
      if (!appPathRoutes[normalizedPath]) {
        appPathRoutes[normalizedPath] = [];
      }
      appPathRoutes[normalizedPath].push(entry);
    }
    return appPathRoutes;
  }
  getOriginalAppPaths(route) {
    const originalAppPath = this.appPathRoutes?.[route];
    if (!originalAppPath) {
      return void 0;
    }
    return originalAppPath;
  }
  async renderPageComponent(ctx, bubbleNoFallback) {
    const { query, pathname } = ctx;
    const appPaths = this.getOriginalAppPaths(pathname);
    const isAppPath = Array.isArray(appPaths);
    let page = pathname;
    if (isAppPath) {
      page = appPaths[0];
    }
    const result = await this.findPageComponents({
      pathname: page,
      query,
      params: ctx.renderOpts.params ?? {},
      isAppPath,
      appPaths,
      sriEnabled: Boolean(this.nextConfig.experimental?.sri?.algorithm)
    });
    if (result) {
      try {
        return await this.renderToResponseWithComponents(ctx, result);
      } catch (error) {
        const isNoFallbackError = error instanceof NoFallbackError;
        if (!isNoFallbackError || isNoFallbackError && bubbleNoFallback) {
          throw error;
        }
      }
    }
    return false;
  }
  async pipe(fn, partialContext) {
    const isBotRequest = isBot(partialContext.req.headers["user-agent"] ?? "");
    const ctx = {
      ...partialContext,
      renderOpts: {
        ...this.renderOpts,
        supportsDynamicHTML: !isBotRequest
      }
    };
    const payload = await fn(ctx);
    if (!payload) {
      return;
    }
    const { req, res } = ctx;
    const { body, type, revalidateOptions } = payload;
    if (!res.sent) {
      const { generateEtags, poweredByHeader, dev } = this.renderOpts;
      if (dev) {
        res.setHeader("Cache-Control", "no-store, must-revalidate");
      }
      return this.sendRenderResult(req, res, {
        result: body,
        type,
        generateEtags,
        poweredByHeader,
        options: revalidateOptions
      });
    }
  }
  async renderErrorToResponse(ctx, error) {
    const { res, query } = ctx;
    try {
      let result;
      const is404 = res.statusCode === 404;
      let using404Page = false;
      if (is404 && await this.pageChecker.hasPage("/404")) {
        result = await this.findPageComponents({
          pathname: "/404",
          query,
          params: {},
          isAppPath: false
        });
        using404Page = result !== null;
      }
      let statusPage = `/${res.statusCode}`;
      if (!ctx.query.__nextCustomErrorRender && !result && STATIC_STATUS_PAGES.includes(statusPage) && (statusPage !== "/500" || !this.renderOpts.dev)) {
        result = await this.findPageComponents({
          pathname: statusPage,
          query,
          params: {},
          isAppPath: false
        });
      }
      if (!result) {
        result = await this.findPageComponents({
          pathname: "/_error",
          query,
          params: {},
          isAppPath: false
        });
        statusPage = "/_error";
      }
      if (!using404Page && await this.pageChecker.hasPage("/_error") && !await this.pageChecker.hasPage("/404")) {
        this.customErrorNo404Warn();
      }
      try {
        return await this.renderToResponseWithComponents(
          {
            ...ctx,
            pathname: statusPage,
            renderOpts: {
              ...ctx.renderOpts,
              err: error
            }
          },
          result
        );
      } catch (maybeFallbackError) {
        if (maybeFallbackError instanceof NoFallbackError) {
          throw new TypeError("invariant: failed to render error page");
        }
        throw maybeFallbackError;
      }
    } catch (error2) {
      const renderToHtmlError = getProperError(error2);
      const isWrappedError = renderToHtmlError instanceof WrappedBuildError;
      if (!isWrappedError) {
        this.logger.logError(renderToHtmlError);
      }
      res.statusCode = 500;
      const fallbackComponents = await this.getFallbackErrorComponents();
      if (fallbackComponents) {
        return this.renderToResponseWithComponents(
          {
            ...ctx,
            pathname: "/_error",
            renderOpts: {
              ...ctx.renderOpts,
              err: isWrappedError ? renderToHtmlError.innerError : renderToHtmlError
            }
          },
          {
            query,
            components: fallbackComponents
          }
        );
      }
      return {
        type: "html",
        body: RenderResult.fromStatic("Internal Server Error")
      };
    }
  }
  async renderToResponseWithComponents({ req, res, pathname, renderOpts: options }, { components, query }) {
    const is404Page = pathname === "/404";
    const is500Page = pathname === "/500";
    const { isAppPath } = components;
    const isLikeServerless = typeof components.ComponentMod === "object" && typeof components.ComponentMod.renderReqToHTML === "function";
    const hasServerProps = Boolean(components.getServerSideProps);
    let hasStaticPaths = Boolean(components.getStaticPaths);
    const hasGetInitialProps = Boolean(components.Component?.getInitialProps);
    let isSSG = Boolean(components.getStaticProps);
    let urlPathname = parseUrl(req.url || "").pathname ?? "/";
    let resolvedUrlPathname = getRequestMeta(req, "_nextRewroteUrl") ?? urlPathname;
    let staticPaths;
    let fallbackMode;
    if (isAppPath) {
      const pathsResult = await this.getStaticPaths({
        pathname,
        originalAppPath: components.pathname
      });
      staticPaths = pathsResult.staticPaths;
      fallbackMode = pathsResult.fallbackMode;
      const hasFallback = typeof fallbackMode !== "undefined";
      if (hasFallback) {
        hasStaticPaths = true;
      }
      if (hasFallback || staticPaths?.includes(resolvedUrlPathname)) {
        isSSG = true;
      } else if (!this.renderOpts.dev) {
        const manifest = this.manifestProvider.getPrerenderManifest();
        isSSG = isSSG || Boolean(manifest.routes[pathname === "/index" ? "/" : pathname]);
      }
    }
    let isDataRequest = Boolean(query.__nextDataReq) && (isSSG || hasServerProps);
    if (isAppPath && req.headers.__rsc__ && isSSG) {
      isDataRequest = true;
      delete req.headers.__rsc__;
      delete req.headers.__next_router_state_tree__;
      delete req.headers.__next_router_prefetch__;
    }
    delete query.__nextDataReq;
    if (Boolean(req.headers["x-nextjs-data"]) && (!res.statusCode || res.statusCode === 200)) {
      res.setHeader(
        "x-nextjs-matched-path",
        `${query.__nextLocale ? `/${query.__nextLocale}` : ""}${pathname}`
      );
    }
    const isFlightRequest = Boolean(
      this.manifestProvider.getServerComponentManifest() && req.headers.__rsc__
    );
    if (is404Page && !isDataRequest && !isFlightRequest) {
      res.statusCode = 404;
    }
    if (STATIC_STATUS_PAGES.includes(pathname)) {
      res.statusCode = Number.parseInt(pathname.slice(1), 10);
    }
    if (!is404Page && !is500Page && pathname !== "/_error" && req.method !== "HEAD" && req.method !== "GET" && (typeof components.Component === "string" || isSSG)) {
      res.statusCode = 405;
      res.setHeader("Allow", ["GET", "HEAD"]);
      await this.renderError(void 0, req, res, pathname);
      return void 0;
    }
    if (typeof components.Component === "string") {
      return {
        type: "html",
        body: RenderResult.fromStatic(components.Component)
      };
    }
    if (!query.amp) {
      delete query.amp;
    }
    if (options.supportsDynamicHTML === true) {
      const isBotRequest = isBot(req.headers["user-agent"] ?? "");
      const isSupportedDocument = typeof components.Document?.getInitialProps !== "function" || Boolean(process.env.__NEXT_REACT_ROOT) && NEXT_BUILTIN_DOCUMENT in components.Document;
      options.supportsDynamicHTML = !isSSG && !isLikeServerless && !isBotRequest && !query.amp && isSupportedDocument;
    }
    const defaultLocale = isSSG ? this.nextConfig.i18n?.defaultLocale : query.__nextDefaultLocale;
    const locale = query.__nextLocale;
    const locales = this.nextConfig.i18n?.locales;
    const isPreviewMode = false;
    let isManualRevalidate = false;
    let revalidateOnlyGenerated = false;
    if (isSSG) {
      ({ isManualRevalidate, revalidateOnlyGenerated } = checkIsManualRevalidate(
        req,
        this.renderOpts.previewProps
      ));
    }
    urlPathname = removeTrailingSlash(urlPathname);
    resolvedUrlPathname = normalizeLocalePath2(
      removeTrailingSlash(resolvedUrlPathname),
      this.nextConfig.i18n?.locales
    ).pathname;
    const handleRedirect = (pageData) => {
      const redirect = {
        destination: pageData.pageProps.__N_REDIRECT,
        statusCode: pageData.pageProps.__N_REDIRECT_STATUS,
        basePath: pageData.pageProps.__N_REDIRECT_BASE_PATH
      };
      const statusCode = getRedirectStatus(redirect);
      const { basePath } = this.nextConfig;
      if (basePath && redirect.basePath !== false && redirect.destination.startsWith("/")) {
        redirect.destination = `${basePath}${redirect.destination}`;
      }
      if (redirect.destination.startsWith("/")) {
        redirect.destination = normalizeRepeatedSlashes(redirect.destination);
      }
      res.redirect(redirect.destination, statusCode).body(redirect.destination).send();
    };
    if (isDataRequest) {
      resolvedUrlPathname = this.stripNextDataPath(resolvedUrlPathname);
      urlPathname = this.stripNextDataPath(urlPathname);
    }
    let ssgCacheKey = isPreviewMode || !isSSG || options.supportsDynamicHTML ? null : `${locale ? `/${locale}` : ""}${(pathname === "/" || resolvedUrlPathname === "/") && locale ? "" : resolvedUrlPathname}${query.amp ? ".amp" : ""}`;
    if ((is404Page || is500Page) && isSSG) {
      ssgCacheKey = `${locale ? `/${locale}` : ""}${pathname}${query.amp ? ".amp" : ""}`;
    }
    if (ssgCacheKey) {
      ssgCacheKey = ssgCacheKey.split("/").map((seg) => {
        try {
          seg = escapePathDelimiters(decodeURIComponent(seg), true);
        } catch {
          throw new DecodeError("failed to decode param");
        }
        return seg;
      }).join("/");
      ssgCacheKey = ssgCacheKey === "/index" && pathname === "/" ? "/" : ssgCacheKey;
    }
    const doRender = async () => {
      let pageData;
      let body;
      let sprRevalidate;
      let isNotFound;
      let isRedirect;
      const origQuery = parseUrl(req.url || "", true).query;
      if (options.params) {
        for (const key of Object.keys(options.params)) {
          delete origQuery[key];
        }
      }
      const hadTrailingSlash = urlPathname !== "/" && this.nextConfig.trailingSlash;
      const resolvedUrl = formatUrl({
        pathname: `${resolvedUrlPathname}${hadTrailingSlash ? "/" : ""}`,
        query: origQuery
      });
      const renderOptions = {
        ...components,
        ...options,
        isDataReq: isDataRequest,
        resolvedUrl,
        locale,
        locales,
        defaultLocale,
        resolvedAsPath: hasServerProps || hasGetInitialProps ? formatUrl({
          pathname: `${urlPathname}${hadTrailingSlash ? "/" : ""}`,
          query: origQuery
        }) : resolvedUrl
      };
      if (isSSG || hasStaticPaths) {
        renderOptions.supportsDynamicHTML = false;
      }
      const renderResult = await this.renderHTML(
        req,
        res,
        pathname,
        query,
        renderOptions
      );
      body = renderResult;
      pageData = renderOptions.pageData;
      sprRevalidate = renderOptions.revalidate;
      isNotFound = renderOptions.isNotFound;
      isRedirect = renderOptions.isRedirect;
      let value;
      if (isNotFound) {
        value = void 0;
      } else if (isRedirect) {
        value = { kind: "REDIRECT", props: pageData };
      } else {
        if (!body) {
          return void 0;
        }
        value = { kind: "PAGE", html: body, pageData };
      }
      return { revalidate: sprRevalidate, value: value ?? null };
    };
    const cacheEntry = await this.responseCache.get(
      ssgCacheKey,
      async (hasResolved, hadCache) => {
        const isProduction = !this.renderOpts.dev;
        const isDynamicPathname = isDynamicRoute(pathname);
        const didRespond = hasResolved || res.sent;
        if (!staticPaths) {
          ({ staticPaths, fallbackMode } = hasStaticPaths ? await this.getStaticPaths({ pathname }) : { staticPaths: void 0, fallbackMode: false });
        }
        if (fallbackMode === "static" && isBot(req.headers["user-agent"] || "")) {
          fallbackMode = "blocking";
        }
        if (isManualRevalidate && revalidateOnlyGenerated && !hadCache) {
          await this.render404(req, res);
          return null;
        }
        if (isManualRevalidate && (fallbackMode !== false || hadCache)) {
          fallbackMode = "blocking";
        }
        if (process.env.NEXT_RUNTIME !== "edge" && fallbackMode !== "blocking" && ssgCacheKey && !didRespond && !isPreviewMode && isDynamicPathname && (isProduction || !staticPaths || !staticPaths.includes(
          query.amp ? ssgCacheKey.replace(/\.amp$/, "") : ssgCacheKey
        ))) {
          if ((isProduction || staticPaths) && fallbackMode !== "static") {
            throw new NoFallbackError();
          }
          if (!isDataRequest) {
            if (isProduction) {
              const html = await this.getFallback(
                locale ? `/${locale}${pathname}` : pathname
              );
              return {
                value: {
                  kind: "PAGE",
                  html: RenderResult.fromStatic(html),
                  pageData: {}
                }
              };
            }
            query.__nextFallback = "true";
            const result2 = await doRender();
            if (!result2) {
              return null;
            }
            delete result2.revalidate;
            return result2;
          }
        }
        const result = await doRender();
        if (!result) {
          return null;
        }
        return {
          ...result,
          revalidate: result.revalidate !== void 0 ? result.revalidate : 1
        };
      },
      {
        isManualRevalidate,
        isPrefetch: req.headers.purpose === "prefetch"
      }
    );
    if (!cacheEntry) {
      if (ssgCacheKey && !(isManualRevalidate && revalidateOnlyGenerated)) {
        throw new Error("invariant: cache entry required but not generated");
      }
      return void 0;
    }
    if (isSSG) {
      res.setHeader(
        "x-nextjs-cache",
        isManualRevalidate ? "REVALIDATED" : cacheEntry.isMiss ? "MISS" : cacheEntry.isStale ? "STALE" : "HIT"
      );
    }
    const { revalidate, value: cachedData } = cacheEntry;
    const revalidateOptions = typeof revalidate !== "undefined" && (!this.renderOpts.dev || hasServerProps && !isDataRequest) ? {
      private: isPreviewMode || is404Page && cachedData,
      stateful: !isSSG,
      revalidate
    } : void 0;
    if (!cachedData) {
      if (revalidateOptions) {
        setRevalidateHeaders(res, revalidateOptions);
      }
      if (isDataRequest) {
        res.statusCode = 404;
        res.body('{"notFound":true}').send();
        return void 0;
      }
      if (this.renderOpts.dev) {
        query.__nextNotFoundSrcPage = pathname;
      }
      await this.render404(
        req,
        res,
        {
          pathname,
          query
        },
        false
      );
      return void 0;
    }
    if (cachedData.kind === "REDIRECT") {
      if (revalidateOptions) {
        setRevalidateHeaders(res, revalidateOptions);
      }
      if (isDataRequest) {
        return {
          type: "json",
          body: RenderResult.fromStatic(
            JSON.stringify(cachedData.props)
          ),
          revalidateOptions
        };
      }
      handleRedirect(cachedData.props);
      return void 0;
    }
    if (cachedData.kind === "IMAGE") {
      throw new Error("invariant SSG should not return an image cache value");
    } else {
      return {
        type: isDataRequest ? isAppPath ? "rsc" : "json" : "html",
        body: isDataRequest ? RenderResult.fromStatic(
          isAppPath ? cachedData.pageData : JSON.stringify(cachedData.pageData)
        ) : cachedData.html,
        revalidateOptions
      };
    }
  }
  stripNextDataPath(path, stripLocale = true) {
    if (path.includes(this.manifestProvider.getBuildId())) {
      const splitPath = path.slice(
        Math.max(
          0,
          path.indexOf(this.manifestProvider.getBuildId()) + this.manifestProvider.getBuildId().length
        )
      );
      path = denormalizePagePath2(splitPath.replace(/\.json$/, ""));
    }
    if (this.nextConfig.i18n && stripLocale) {
      const { locales } = this.nextConfig.i18n;
      return normalizeLocalePath2(path, locales).pathname;
    }
    return path;
  }
  async renderToResponse(ctx) {
    const { res, query, pathname } = ctx;
    let page = pathname;
    const bubbleNoFallback = Boolean(query._nextBubbleNoFallback);
    delete query._nextBubbleNoFallback;
    try {
      if (!isDynamicRoute(page)) {
        const result = await this.renderPageComponent(ctx, bubbleNoFallback);
        if (result !== false)
          return result;
      }
      if (this.dynamicRoutes) {
        for (const dynamicRoute of this.dynamicRoutes) {
          const parameters = dynamicRoute.match(pathname);
          if (!parameters) {
            continue;
          }
          page = dynamicRoute.page;
          const result = await this.renderPageComponent(
            {
              ...ctx,
              pathname: page,
              renderOpts: {
                ...ctx.renderOpts,
                params: parameters
              }
            },
            bubbleNoFallback
          );
          if (result !== false)
            return result;
        }
      }
    } catch (error) {
      const error_ = getProperError(error);
      if (error instanceof MissingStaticPage) {
        console.error(
          "Invariant: failed to load static page",
          JSON.stringify(
            {
              page,
              url: ctx.req.url,
              matchedPath: ctx.req.headers["x-matched-path"],
              initUrl: getRequestMeta(ctx.req, "__NEXT_INIT_URL"),
              didRewrite: getRequestMeta(ctx.req, "_nextDidRewrite"),
              rewroteUrl: getRequestMeta(ctx.req, "_nextRewroteUrl")
            },
            null,
            2
          )
        );
        throw error_;
      }
      if (error_ instanceof NoFallbackError && bubbleNoFallback) {
        throw error_;
      }
      if (error_ instanceof DecodeError || error_ instanceof NormalizeError) {
        res.statusCode = 400;
        return this.renderErrorToResponse(ctx, error_);
      }
      res.statusCode = 500;
      if (await this.pageChecker.hasPage("/500")) {
        ctx.query.__nextCustomErrorRender = "1";
        await this.renderErrorToResponse(ctx, error_);
        delete ctx.query.__nextCustomErrorRender;
      }
      const isWrappedError = error_ instanceof WrappedBuildError;
      if (!isWrappedError) {
        this.logger.logError(getProperError(error_));
      }
      const response = await this.renderErrorToResponse(
        ctx,
        isWrappedError ? error_.innerError : error_
      );
      return response;
    }
    if (this.hasMiddleware && Boolean(ctx.req.headers["x-nextjs-data"]) && (!res.statusCode || res.statusCode === 200 || res.statusCode === 404)) {
      res.setHeader(
        "x-nextjs-matched-path",
        `${query.__nextLocale ? `/${query.__nextLocale}` : ""}${pathname}`
      );
      res.statusCode = 200;
      res.setHeader("content-type", "application/json");
      res.body("{}");
      res.send();
      return void 0;
    }
    res.statusCode = 404;
    return this.renderErrorToResponse(ctx, void 0);
  }
};
var IgnextRenderer = class extends Renderer {
  loadComponent;
  constructor({ loadComponent, ...options }) {
    super(options);
    this.loadComponent = loadComponent;
  }
  async findPageComponents({
    pathname,
    query,
    params
  }) {
    const result = await this.loadComponent(pathname);
    if (!result)
      return void 0;
    return {
      query: {
        ...query,
        ...params
      },
      components: result
    };
  }
  async sendRenderResult(request, res, options) {
    res.setHeader("X-Edge-Runtime", "1");
    if (options.poweredByHeader && options.type === "html") {
      res.setHeader("X-Powered-By", "Next.js");
    }
    const resultContentType = options.result.contentType();
    if (!res.getHeader("Content-Type")) {
      res.setHeader(
        "Content-Type",
        resultContentType ? resultContentType : options.type === "json" ? "application/json" : "text/html; charset=utf-8"
      );
    }
    if (options.result.isDynamic()) {
      const writer = res.transformStream.writable.getWriter();
      await options.result.pipe({
        write: (chunk) => writer.write(chunk),
        end: () => writer.close(),
        _destroy: (error) => writer.abort(error),
        get destroy() {
          return this._destroy;
        },
        set destroy(value) {
          this._destroy = value;
        },
        cork() {
        },
        uncork() {
        }
      });
    } else {
      const payload = options.result.toUnchunkedString();
      res.setHeader("Content-Length", String(byteLength(payload)));
      if (options.generateEtags) {
        res.setHeader("ETag", generateETag(payload));
      }
      res.body(payload);
    }
    res.send();
  }
  async getFallback(page) {
    return "";
  }
  async renderHTML(request, res, pathname, query, renderOptions) {
    const renderFunction = renderOptions.isAppPath ? renderToHTMLOrFlight : renderToHTML;
    return await renderFunction(
      {
        url: request.url,
        cookies: request.cookies,
        headers: request.headers
      },
      {},
      pathname,
      query,
      Object.assign(renderOptions, {
        disableOptimizedLoading: true,
        runtime: "experimental-edge"
      })
    ) ?? void 0;
  }
};

// src/internal/router-builder.ts
import isError, { getProperError as getProperError2 } from "next/dist/esm/lib/is-error";
import {
  NoFallbackError as NoFallbackError2
} from "next/dist/esm/server/base-server";
import { requestToBodyStream } from "next/dist/esm/server/body-streams";
import { addRequestMeta, getRequestMeta as getRequestMeta2 } from "next/dist/esm/server/request-meta";
import Router from "next/dist/esm/server/router";
import {
  createHeaderRoute,
  createRedirectRoute,
  getCustomRoute
} from "next/dist/esm/server/server-route-utils";
import { adapter } from "next/dist/esm/server/web/adapter";
import { toNodeHeaders } from "next/dist/esm/server/web/utils";
import { CLIENT_STATIC_FILES_RUNTIME } from "next/dist/esm/shared/lib/constants";
import { detectDomainLocale } from "next/dist/esm/shared/lib/i18n/detect-domain-locale";
import { normalizeLocalePath as normalizeLocalePath3 } from "next/dist/esm/shared/lib/i18n/normalize-locale-path";
import { getNextPathnameInfo } from "next/dist/esm/shared/lib/router/utils/get-next-pathname-info";
import getRouteFromAssetPath from "next/dist/esm/shared/lib/router/utils/get-route-from-asset-path";
import { isDynamicRoute as isDynamicRoute2 } from "next/dist/esm/shared/lib/router/utils/is-dynamic";
import {
  getMiddlewareRouteMatcher
} from "next/dist/esm/shared/lib/router/utils/middleware-route-matcher";
import { parseUrl as parseUrl2 } from "next/dist/esm/shared/lib/router/utils/parse-url";
import { getPathMatch } from "next/dist/esm/shared/lib/router/utils/path-match";
import { prepareDestination } from "next/dist/esm/shared/lib/router/utils/prepare-destination";
import { urlQueryToSearchParams } from "next/dist/esm/shared/lib/router/utils/querystring";
import { relativizeURL } from "next/dist/esm/shared/lib/router/utils/relativize-url";
import { removeTrailingSlash as removeTrailingSlash2 } from "next/dist/esm/shared/lib/router/utils/remove-trailing-slash";
import { DecodeError as DecodeError2 } from "next/dist/esm/shared/lib/utils";
import { NextResponse } from "next/server";
var RouterBuilder = class {
  router;
  getRouter() {
    if (this.router) {
      return this.router;
    }
    this.router = new Router(this.build());
    return this.router;
  }
};
var IgnextRouterBuilder = class extends RouterBuilder {
  constructor(manifestProvider, nextConfig, renderer, customRoutes, dynamicRoutes, pageChecker, loadFunction, env) {
    super();
    this.manifestProvider = manifestProvider;
    this.nextConfig = nextConfig;
    this.renderer = renderer;
    this.customRoutes = customRoutes;
    this.dynamicRoutes = dynamicRoutes;
    this.pageChecker = pageChecker;
    this.loadFunction = loadFunction;
    this.env = env;
  }
  generatePublicRoutes() {
    return [];
  }
  generateImageRoutes() {
    return [
      {
        match: getPathMatch("/_next/image"),
        type: "route",
        name: "_next/image catchall",
        fn: async (request, response, _parameters, _parsedUrl) => {
          const imagesConfig = this.nextConfig.images;
          if (imagesConfig?.loader !== "default") {
            await this.renderer.render404(request, response);
            return { finished: true };
          }
          return { finished: true };
        }
      }
    ];
  }
  generateFsStaticRoutes() {
    return [
      {
        match: getPathMatch("/_next/static/:path*"),
        type: "route",
        name: "_next/static catchall",
        fn: async (request, response, parameters, parsedUrl) => {
          if (!parameters.path) {
            await this.renderer.render404(request, response, parsedUrl);
            return {
              finished: true
            };
          }
          if (parameters.path[0] === CLIENT_STATIC_FILES_RUNTIME || parameters.path[0] === "chunks" || parameters.path[0] === "css" || parameters.path[0] === "image" || parameters.path[0] === "media" || parameters.path[0] === this.manifestProvider.getBuildId() || parameters.path[0] === "pages" || parameters.path[1] === "pages") {
            response.setHeader(
              "Cache-Control",
              "public, max-age=31536000, immutable"
            );
          }
          if (!(request.method === "GET" || request.method === "HEAD")) {
            response.statusCode = 405;
            response.setHeader("Allow", ["GET", "HEAD"]);
            return this.renderer.renderError(
              void 0,
              request,
              response,
              parameters.path
            );
          }
          const _response = await this.env.ASSETS.fetch(
            request.request
          );
          if (!_response.ok) {
            return { finished: false };
          }
          response.statusCode = _response.status;
          response.statusMessage = _response.statusText;
          _response.headers.forEach((value, key) => {
            if (key.toLowerCase() === "set-cookie") {
              response.setHeader(key, value);
            } else {
              response.appendHeader(key, value);
            }
          });
          if (_response.body) {
            _response.body.pipeThrough(
              response.transformStream
            );
          }
          response.send();
          return {
            finished: true
          };
        }
      }
    ];
  }
  generateRewrites({
    restrictedRedirectPaths
  }) {
    let beforeFiles = [];
    let afterFiles = [];
    let fallback = [];
    const buildRewrite = (rewrite, check = true) => {
      const rewriteRoute = getCustomRoute({
        type: "rewrite",
        rule: rewrite,
        restrictedRedirectPaths
      });
      return {
        ...rewriteRoute,
        check,
        type: rewriteRoute.type,
        name: `Rewrite route ${rewriteRoute.source}`,
        match: rewriteRoute.match,
        matchesBasePath: true,
        matchesLocale: true,
        matchesLocaleAPIRoutes: true,
        matchesTrailingSlash: true,
        async fn(request, res, parameters, parsedUrl, upgradeHead) {
          const { newUrl, parsedDestination } = prepareDestination({
            appendParamsToQuery: true,
            destination: rewriteRoute.destination,
            params: parameters,
            query: parsedUrl.query
          });
          if (parsedDestination.protocol) {
            throw new Error("Do not support proxy request yet.");
          }
          addRequestMeta(request, "_nextRewroteUrl", newUrl);
          addRequestMeta(request, "_nextDidRewrite", newUrl !== request.url);
          return {
            finished: false,
            pathname: newUrl,
            query: parsedDestination.query
          };
        }
      };
    };
    if (Array.isArray(this.customRoutes.rewrites)) {
      afterFiles = this.customRoutes.rewrites.map((r) => buildRewrite(r));
    } else {
      beforeFiles = this.customRoutes.rewrites.beforeFiles.map(
        (r) => buildRewrite(r, false)
      );
      afterFiles = this.customRoutes.rewrites.afterFiles.map(
        (r) => buildRewrite(r)
      );
      fallback = this.customRoutes.rewrites.fallback.map(
        (r) => buildRewrite(r)
      );
    }
    return {
      beforeFiles,
      afterFiles,
      fallback
    };
  }
  build() {
    const publicRoutes = this.generatePublicRoutes();
    const imageRoutes = this.generateImageRoutes();
    const fsRoutes = [
      ...this.generateFsStaticRoutes(),
      {
        match: getPathMatch("/_next/data/:path*"),
        type: "route",
        name: "_next/data catchall",
        check: true,
        fn: async (request, res, parameters, _parsedUrl) => {
          if (!parameters.path || parameters.path[0] !== this.manifestProvider.getBuildId()) {
            await this.renderer.render404(request, res, _parsedUrl);
            return {
              finished: true
            };
          }
          parameters.path.shift();
          const lastParameter = parameters.path[parameters.path.length - 1];
          if (typeof lastParameter !== "string" || !lastParameter.endsWith(".json")) {
            await this.renderer.render404(request, res, _parsedUrl);
            return {
              finished: true
            };
          }
          let pathname = `/${parameters.path.join("/")}`;
          pathname = getRouteFromAssetPath(pathname, ".json");
          if (this.router.catchAllMiddleware[0]) {
            if (this.nextConfig.trailingSlash && !pathname.endsWith("/")) {
              pathname += "/";
            }
            if (!this.nextConfig.trailingSlash && pathname.length > 1 && pathname.endsWith("/")) {
              pathname = pathname.slice(0, Math.max(0, pathname.length - 1));
            }
          }
          if (this.nextConfig.i18n) {
            const { host } = request?.headers || {};
            const hostname = host?.split(":")[0].toLowerCase();
            const localePathResult = normalizeLocalePath3(
              pathname,
              this.nextConfig.i18n.locales
            );
            const { defaultLocale } = detectDomainLocale(this.nextConfig.i18n.domains, hostname) ?? {};
            let detectedLocale = "";
            if (localePathResult.detectedLocale) {
              pathname = localePathResult.pathname;
              detectedLocale = localePathResult.detectedLocale;
            }
            _parsedUrl.query.__nextLocale = detectedLocale;
            _parsedUrl.query.__nextDefaultLocale = defaultLocale ?? this.nextConfig.i18n.defaultLocale;
            if (!detectedLocale && !this.router.catchAllMiddleware[0]) {
              _parsedUrl.query.__nextLocale = _parsedUrl.query.__nextDefaultLocale;
              await this.renderer.render404(request, res, _parsedUrl);
              return { finished: true };
            }
          }
          return {
            pathname,
            query: { ..._parsedUrl.query, __nextDataReq: "1" },
            finished: false
          };
        }
      },
      ...imageRoutes,
      {
        match: getPathMatch("/_next/:path*"),
        type: "route",
        name: "_next catchall",
        fn: async (request, res, _parameters, parsedUrl) => {
          await this.renderer.render404(request, res, parsedUrl);
          return {
            finished: true
          };
        }
      },
      ...publicRoutes
    ];
    const restrictedRedirectPaths = this.nextConfig.basePath ? [`${this.nextConfig.basePath}/_next`] : ["/_next"];
    const headers = this.customRoutes.headers.map(
      (rule) => createHeaderRoute({ rule, restrictedRedirectPaths })
    );
    const redirects = this.customRoutes.redirects.map(
      (rule) => createRedirectRoute({ rule, restrictedRedirectPaths })
    );
    const rewrites = this.generateRewrites({ restrictedRedirectPaths });
    const catchAllMiddleware = this.generateCatchAllMiddlewareRoute();
    const catchAllRoute = {
      match: getPathMatch("/:path*"),
      type: "route",
      matchesLocale: true,
      name: "Catchall render",
      fn: async (request, res, _parameters, parsedUrl) => {
        let { pathname, query } = parsedUrl;
        if (!pathname) {
          throw new Error("pathname is undefined");
        }
        pathname = removeTrailingSlash2(pathname);
        if (this.nextConfig.i18n) {
          const localePathResult = normalizeLocalePath3(
            pathname,
            this.nextConfig.i18n?.locales
          );
          if (localePathResult.detectedLocale) {
            pathname = localePathResult.pathname;
            parsedUrl.query.__nextLocale = localePathResult.detectedLocale;
          }
        }
        const bubbleNoFallback = Boolean(query._nextBubbleNoFallback);
        if (pathname === "/api" || pathname?.startsWith("/api/")) {
          delete query._nextBubbleNoFallback;
          const handled = await this.handleApiRequest(
            request,
            res,
            pathname,
            query
          );
          if (handled) {
            return { finished: true };
          }
        }
        try {
          await this.renderer.render(
            request,
            res,
            pathname,
            query,
            parsedUrl,
            true
          );
          return {
            finished: true
          };
        } catch (error) {
          if (error instanceof NoFallbackError2 && bubbleNoFallback) {
            return {
              finished: false
            };
          }
          throw error;
        }
      }
    };
    return {
      headers,
      fsRoutes,
      rewrites,
      redirects,
      catchAllRoute,
      catchAllMiddleware,
      useFileSystemPublicRoutes: true,
      dynamicRoutes: this.dynamicRoutes,
      pageChecker: this.pageChecker.hasPage.bind(this.pageChecker),
      nextConfig: this.nextConfig
    };
  }
  async handleApiRequest(request, res, pathname, query) {
    let page = pathname;
    let parameters;
    let pageFound = !isDynamicRoute2(page) && await this.pageChecker.hasPage(page);
    if (!pageFound && this.dynamicRoutes) {
      for (const dynamicRoute of this.dynamicRoutes) {
        parameters = dynamicRoute.match(pathname) || void 0;
        if (dynamicRoute.page.startsWith("/api") && parameters) {
          page = dynamicRoute.page;
          pageFound = true;
          break;
        }
      }
    }
    if (!pageFound) {
      return false;
    }
    return this.runApi(request, res, query, parameters, page);
  }
  async runApi(request, res, query, parameters, page) {
    const func = await this.loadFunction(page);
    if (!func) {
      return false;
    }
    const isDataRequest = Boolean(query.__nextDataReq);
    const initialUrl = new URL(
      getRequestMeta2(request, "__NEXT_INIT_URL") ?? "/",
      "http://n"
    );
    const queryString = urlQueryToSearchParams({
      ...Object.fromEntries(initialUrl.searchParams),
      ...query,
      ...parameters
    }).toString();
    if (isDataRequest) {
      request.headers["x-nextjs-data"] = "1";
    }
    initialUrl.search = queryString;
    const url = initialUrl.toString();
    if (!url.startsWith("http")) {
      throw new Error(
        "To use middleware you must provide a `hostname` and `port` to the Next.js Server"
      );
    }
    const cloned = ["HEAD", "GET"].includes(request.method) ? void 0 : getRequestMeta2(request, "__NEXT_CLONABLE_BODY")?.cloneBodyStream();
    const result = await adapter({
      handler: func,
      page,
      request: {
        headers: request.headers,
        method: request.method,
        nextConfig: {
          basePath: this.nextConfig.basePath,
          i18n: this.nextConfig.i18n,
          trailingSlash: this.nextConfig.trailingSlash
        },
        url,
        page: {
          name: page,
          ...parameters && { params: parameters }
        },
        body: cloned ? requestToBodyStream({ ReadableStream }, Uint8Array, cloned) : void 0
      }
    });
    res.statusCode = result.response.status;
    res.statusMessage = result.response.statusText;
    result.response.headers.forEach((value, key) => {
      if (key.toLowerCase() === "set-cookie") {
        res.setHeader(key, value);
      } else {
        res.appendHeader(key, value);
      }
    });
    if (result.response.body) {
      result.response.body.pipeThrough(res.transformStream);
    }
    res.send();
    return true;
  }
  generateCatchAllMiddlewareRoute() {
    const routes = [];
    if (this.getMiddleware()) {
      const middlewareCatchAllRoute = {
        match: getPathMatch("/:path*"),
        matchesBasePath: true,
        matchesLocale: true,
        type: "route",
        name: "middleware catchall",
        fn: async (request, res, _parameters, parsed) => {
          const middleware = this.getMiddleware();
          if (!middleware) {
            return { finished: false };
          }
          const initUrl = getRequestMeta2(request, "__NEXT_INIT_URL");
          const parsedUrl = parseUrl2(initUrl);
          const pathnameInfo = getNextPathnameInfo(parsedUrl.pathname, {
            nextConfig: this.nextConfig
          });
          parsedUrl.pathname = pathnameInfo.pathname;
          const normalizedPathname = removeTrailingSlash2(parsed.pathname ?? "");
          if (!middleware.match(normalizedPathname, request, parsedUrl.query)) {
            return { finished: false };
          }
          let result;
          try {
            result = await this.runMiddleware({
              request,
              response: res,
              parsedUrl,
              parsed
            });
          } catch (error_) {
            if (isError(error_) && error_.code === "ENOENT") {
              await this.renderer.render404(request, res, parsed);
              return { finished: true };
            }
            if (error_ instanceof DecodeError2) {
              res.statusCode = 400;
              await this.renderer.renderError(
                error_,
                request,
                res,
                parsed.pathname ?? ""
              );
              return { finished: true };
            }
            const error = getProperError2(error_);
            console.error(error);
            res.statusCode = 500;
            await this.renderer.renderError(
              error,
              request,
              res,
              parsed.pathname ?? ""
            );
            return { finished: true };
          }
          if ("finished" in result) {
            return result;
          }
          if (result.response.headers.has("x-middleware-rewrite")) {
            const value = result.response.headers.get("x-middleware-rewrite");
            const rel = relativizeURL(value, initUrl);
            result.response.headers.set("x-middleware-rewrite", rel);
          }
          if (result.response.headers.has("Location")) {
            const value = result.response.headers.get("Location");
            const rel = relativizeURL(value, initUrl);
            result.response.headers.set("Location", rel);
          }
          if (!result.response.headers.has("x-middleware-rewrite") && !result.response.headers.has("x-middleware-next") && !result.response.headers.has("Location")) {
            result.response.headers.set("x-middleware-refresh", "1");
          }
          result.response.headers.delete("x-middleware-next");
          for (const [key, value] of Object.entries(
            toNodeHeaders(result.response.headers)
          )) {
            if ([
              "x-middleware-rewrite",
              "x-middleware-redirect",
              "x-middleware-refresh"
            ].includes(key)) {
              continue;
            }
            if (key !== "content-encoding" && value !== void 0) {
              res.setHeader(key, value);
            }
          }
          res.statusCode = result.response.status;
          res.statusMessage = result.response.statusText;
          const location = result.response.headers.get("Location");
          if (location) {
            res.statusCode = result.response.status;
            if (res.statusCode === 308) {
              res.setHeader("Refresh", `0;url=${location}`);
            }
            res.body(location).send();
            return {
              finished: true
            };
          }
          if (result.response.headers.has("x-middleware-rewrite")) {
            const rewritePath = result.response.headers.get(
              "x-middleware-rewrite"
            );
            const parsedDestination = parseUrl2(rewritePath);
            const newUrl = parsedDestination.pathname;
            if (parsedDestination.protocol && (parsedDestination.port ? `${parsedDestination.hostname}:${parsedDestination.port}` : parsedDestination.hostname) !== request.headers.host) {
              throw new Error("Doesn't support proxy request yet");
            }
            if (this.nextConfig.i18n) {
              const localePathResult = normalizeLocalePath3(
                newUrl,
                this.nextConfig.i18n.locales
              );
              if (localePathResult.detectedLocale) {
                parsedDestination.query.__nextLocale = localePathResult.detectedLocale;
              }
            }
            addRequestMeta(request, "_nextRewroteUrl", newUrl);
            addRequestMeta(request, "_nextDidRewrite", newUrl !== request.url);
            return {
              finished: false,
              pathname: newUrl,
              query: parsedDestination.query
            };
          }
          if (result.response.headers.has("x-middleware-refresh")) {
            res.statusCode = result.response.status;
            for await (const chunk of result.response.body ?? []) {
            }
            res.send();
            return {
              finished: true
            };
          }
          return {
            finished: false
          };
        }
      };
      routes.push(middlewareCatchAllRoute);
    }
    return routes;
  }
  getMiddleware() {
    const manifest = this.manifestProvider.getMiddlewareManifest();
    const middleware = manifest?.middleware?.["/"];
    if (!middleware) {
      return;
    }
    return {
      match: getMiddlewareMatcher(middleware),
      page: "/"
    };
  }
  async runMiddleware(parameters) {
    return { finished: false, response: NextResponse.next() };
  }
};
var MiddlewareMatcherCache = /* @__PURE__ */ new WeakMap();
function getMiddlewareMatcher(info) {
  const stored = MiddlewareMatcherCache.get(info);
  if (stored) {
    return stored;
  }
  if (!Array.isArray(info.matchers)) {
    throw new TypeError(
      `Invariant: invalid matchers for middleware ${JSON.stringify(info)}`
    );
  }
  const matcher = getMiddlewareRouteMatcher(info.matchers);
  MiddlewareMatcherCache.set(info, matcher);
  return matcher;
}

// src/internal/server.ts
var IgnextServer = class {
  manifestProvider;
  renderer;
  routeBuilder;
  logger;
  hasAppDir;
  nextConfig;
  constructor(options) {
    this.manifestProvider = options.manifestProvider;
    this.nextConfig = options.nextConfig;
    this.logger = new ConsoleLogger(false);
    this.hasAppDir = Boolean(this.manifestProvider.getAppPathsManifest());
    const dynamicRoutes = this.getDynamicRoutes();
    const pageChecker = new IgnextPageChecker(
      this.manifestProvider,
      this.nextConfig
    );
    this.renderer = new IgnextRenderer({
      nextConfig: this.nextConfig,
      manifestProvider: this.manifestProvider,
      responseCache: new WebResponseCache(true),
      logger: this.logger,
      hasMiddleware: Boolean(
        this.manifestProvider.getMiddlewareManifest()?.middleware?.["/"]
      ),
      renderOpts: {
        poweredByHeader: this.nextConfig.poweredByHeader,
        canonicalBase: this.nextConfig.amp?.canonicalBase ?? "",
        buildId: this.manifestProvider.getBuildId(),
        generateEtags: this.nextConfig.generateEtags,
        previewProps: this.manifestProvider.getPrerenderManifest().preview,
        customServer: false,
        ampOptimizerConfig: this.nextConfig.experimental.amp?.optimizer,
        basePath: this.nextConfig.basePath,
        images: this.nextConfig.images,
        optimizeFonts: this.nextConfig.optimizeFonts,
        fontManifest: this.nextConfig.optimizeFonts ? this.manifestProvider.getFontManifest() : void 0,
        optimizeCss: this.nextConfig.experimental.optimizeCss,
        nextScriptWorkers: this.nextConfig.experimental.nextScriptWorkers,
        disableOptimizedLoading: this.nextConfig.experimental.runtime ? true : this.nextConfig.experimental.disableOptimizedLoading,
        domainLocales: this.nextConfig.i18n?.domains,
        distDir: "",
        runtime: this.nextConfig.experimental.runtime,
        serverComponents: this.hasAppDir,
        crossOrigin: this.nextConfig.crossOrigin ? this.nextConfig.crossOrigin : void 0,
        largePageDataBytes: this.nextConfig.experimental.largePageDataBytes,
        assetPrefix: this.nextConfig.assetPrefix ? this.nextConfig.assetPrefix.replace(/\/$/, "") : ""
      },
      dynamicRoutes,
      pageChecker,
      loadComponent: options.loadComponent
    });
    this.routeBuilder = new IgnextRouterBuilder(
      this.manifestProvider,
      this.nextConfig,
      this.renderer,
      this.getCustomRoutes(),
      dynamicRoutes,
      pageChecker,
      options.loadFunction,
      options.env
    );
  }
  getRequestHandler() {
    return this.handleRequest.bind(this);
  }
  getCustomRoutes() {
    const customRoutes = this.manifestProvider.getRoutesManifest();
    let rewrites;
    if (Array.isArray(customRoutes.rewrites)) {
      rewrites = {
        beforeFiles: [],
        afterFiles: customRoutes.rewrites,
        fallback: []
      };
    } else {
      rewrites = customRoutes.rewrites;
    }
    return Object.assign(customRoutes, { rewrites });
  }
  getDynamicRoutes() {
    const appPathRoutes = {};
    for (const entry of Object.keys(
      this.manifestProvider.getAppPathsManifest() ?? {}
    )) {
      const normalizedPath = normalizeAppPath2(entry) || "/";
      if (!appPathRoutes[normalizedPath]) {
        appPathRoutes[normalizedPath] = [];
      }
      appPathRoutes[normalizedPath].push(entry);
    }
    const addedPages = /* @__PURE__ */ new Set();
    return getSortedRoutes(
      [
        ...Object.keys(appPathRoutes),
        ...Object.keys(this.manifestProvider.getPagesManifest())
      ].map(
        (page) => normalizeLocalePath4(page, this.nextConfig.i18n?.locales).pathname
      )
    ).map((page) => {
      if (addedPages.has(page) || !isDynamicRoute3(page))
        return null;
      addedPages.add(page);
      return {
        page,
        match: getRouteMatcher(getRouteRegex(page))
      };
    }).filter((item) => Boolean(item));
  }
  async handleRequest(request, res, parsedUrl) {
    try {
      const _res = res.originalResponse || res;
      const origSetHeader = _res.setHeader.bind(_res);
      _res.setHeader = (name, value) => {
        if (name.toLowerCase() === "set-cookie") {
          const middlewareValue = getRequestMeta3(
            request,
            "_nextMiddlewareCookie"
          );
          if (!middlewareValue || !Array.isArray(value) || !value.every((item, idx) => item === middlewareValue[idx])) {
            value = [
              ...middlewareValue ?? [],
              ...typeof value === "string" ? [value] : Array.isArray(value) ? value : []
            ];
          }
        }
        return origSetHeader(name, value);
      };
      const urlParts = (request.url || "").split("?");
      const urlNoQuery = urlParts[0];
      if (/(\\|\/\/)/.test(urlNoQuery)) {
        const cleanUrl = normalizeRepeatedSlashes2(request.url);
        res.redirect(cleanUrl, 308).body(cleanUrl).send();
        return;
      }
      setLazyProp(
        { req: request },
        "cookies",
        getCookieParser(request.headers)
      );
      if (!parsedUrl || typeof parsedUrl !== "object") {
        parsedUrl = parseUrl3(request.url, true);
      }
      if (typeof parsedUrl.query === "string") {
        parsedUrl.query = parseQs(parsedUrl.query);
      }
      request.url = normalizeRscPath(request.url, this.hasAppDir);
      parsedUrl.pathname = normalizeRscPath(
        parsedUrl.pathname ?? "",
        this.hasAppDir
      );
      this.attachRequestMeta(request, parsedUrl);
      const domainLocale = detectDomainLocale2(
        this.nextConfig.i18n?.domains,
        getHostname(parsedUrl, request.headers)
      );
      const defaultLocale = domainLocale?.defaultLocale ?? this.nextConfig.i18n?.defaultLocale;
      const url = parseUrlUtil(request.url.replace(/^\/+/, "/"));
      const pathnameInfo = getNextPathnameInfo2(url.pathname, {
        nextConfig: this.nextConfig
      });
      url.pathname = pathnameInfo.pathname;
      if (pathnameInfo.basePath) {
        request.url = removePathPrefix(
          request.url,
          this.nextConfig.basePath ?? ""
        );
        addRequestMeta2(request, "_nextHadBasePath", true);
      }
      addRequestMeta2(
        request,
        "__nextHadTrailingSlash",
        pathnameInfo.trailingSlash
      );
      addRequestMeta2(request, "__nextIsLocaleDomain", Boolean(domainLocale));
      parsedUrl.query.__nextDefaultLocale = defaultLocale;
      if (pathnameInfo.locale) {
        request.url = formatUrl2(url);
        addRequestMeta2(request, "__nextStrippedLocale", true);
      }
      if (!parsedUrl.query.__nextLocale && (pathnameInfo.locale || defaultLocale)) {
        parsedUrl.query.__nextLocale = pathnameInfo.locale ?? defaultLocale;
      }
      if (defaultLocale) {
        const redirect = getLocaleRedirect({
          defaultLocale,
          domainLocale,
          headers: request.headers,
          nextConfig: this.nextConfig,
          pathLocale: pathnameInfo.locale,
          urlParsed: {
            ...url,
            pathname: pathnameInfo.locale ? `/${pathnameInfo.locale}${url.pathname}` : url.pathname
          }
        });
        if (redirect) {
          res.redirect(redirect, TEMPORARY_REDIRECT_STATUS).body(redirect).send();
          return;
        }
      }
      res.statusCode = 200;
      await this.run(request, res, parsedUrl);
      return;
    } catch (error) {
      if (error && typeof error === "object" && error.code === "ERR_INVALID_URL" || error instanceof DecodeError3 || error instanceof NormalizeError2) {
        res.statusCode = 400;
        return this.renderer.renderError(
          void 0,
          request,
          res,
          "/_error",
          {}
        );
      }
      this.logger.logError(getProperError3(error));
      res.statusCode = 500;
      res.body("Internal Server Error").send();
    }
  }
  async run(request, res, parsedUrl) {
    try {
      const matched = await this.routeBuilder.getRouter().execute(request, res, parsedUrl);
      if (matched) {
        return;
      }
    } catch (error) {
      if (error instanceof DecodeError3 || error instanceof NormalizeError2) {
        res.statusCode = 400;
        return this.renderer.renderError(
          void 0,
          request,
          res,
          "/_error",
          {}
        );
      }
      throw error;
    }
    await this.renderer.render404(request, res, parsedUrl);
  }
  attachRequestMeta(request, parsedUrl) {
    const initUrl = request.url;
    addRequestMeta2(request, "__NEXT_INIT_URL", initUrl);
    addRequestMeta2(request, "__NEXT_INIT_QUERY", { ...parsedUrl.query });
    addRequestMeta2(request, "_protocol", "https");
    addRequestMeta2(
      request,
      "__NEXT_CLONABLE_BODY",
      getClonableBody(request.body)
    );
  }
};

// src/internal/adapter.ts
function adapter2(options) {
  return async ({
    request,
    env
  }) => {
    const manifestProvider = new IgnextManifestProvider(
      options.manifestOptions
    );
    const server = new IgnextServer({
      manifestProvider,
      nextConfig: options.config,
      async loadComponent(pathname) {
        const pageOptions = options.pagesOptions[pathname];
        if (!pageOptions) {
          return void 0;
        }
        return {
          dev: options.dev,
          buildManifest: options.manifestOptions.buildManifest,
          reactLoadableManifest: options.manifestOptions.reactLoadableManifest,
          subresourceIntegrityManifest: options.manifestOptions.subresourceIntegrityManifest,
          fontLoaderManifest: options.manifestOptions.fontLoaderManifest,
          Document: options.Document,
          App: options.appMod?.default,
          Component: pageOptions.pageMod.default,
          pageConfig: pageOptions.pageMod.config || {},
          getStaticProps: pageOptions.pageMod.getStaticProps,
          getServerSideProps: pageOptions.pageMod.getServerSideProps,
          getStaticPaths: pageOptions.pageMod.getStaticPaths,
          ComponentMod: pageOptions.pageMod,
          pathname
        };
      },
      async loadFunction(pathname) {
        return options.functionOptions[pathname];
      },
      env
    });
    const handler = server.getRequestHandler();
    transformRequest(request, options.config);
    const extendedRequest = new WebNextRequest(request);
    const extendedResponse = new WebNextResponse();
    handler(extendedRequest, extendedResponse);
    return extendedResponse.toResponse();
  };
}
function transformRequest(request, nextConfig) {
  const { cf } = request;
  const geo = cf && "country" in cf && cf.country !== "T1" ? {
    country: cf.country,
    city: cf.city,
    region: cf.region,
    latitude: cf.latitude,
    longitude: cf.longitude
  } : {};
  Object.assign(request, {
    ip: request.headers.get("CF-Connecting-IP") ?? void 0,
    geo,
    nextConfig
  });
}
export {
  adapter2 as adapter
};
//# sourceMappingURL=adapter.js.map
